<?php
/**
 * @file
 * Drush include for integrating Compass compile.
 */

/**
 * Implements hook_drush_help().
 */
function compass_drush_help($section) {
  switch ($section) {
    case 'drush:compass-compile':
      return dt("Compile a Compass-enabled themes's Sass stylesheets.");
  }
}

/**
 * Implements hook_drush_command().
 */
function compass_drush_command() {
  $items['compass-compile'] = array(
    'description' => "Compiles a compass theme's stylesheets.",
    'arguments' => array(
      'theme_name(s)' => 'A space delimited list of themes, if left blank will compile all themes.',
    ),
    'options' => array(
      '--environment' => 'An environment name to pass to compass, defaults to "production".',
      '--force' => 'Forces a rebuild.',
      '--verbose' => 'Displays compilation output.',
    ),
    'examples' => array(
      'drush compass-compile itvp --environment=development' => 'Compiles the itvp theme stylesheets for the development environment.',
      'drush compass-compile itvp --force' => 'Forces a rebuild of the itvp theme stylesheets for the production (default) environment.',
    ),
    'aliases' => array('sass'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
  );

  return $items;
}

/**
 * Implements drush_hook_COMMAND_validate().
 */
function drush_compass_compile_validate() {
  if (!drush_shell_exec("command -v compass 2>&1")) {
    return drush_set_error('COMPASS', dt('Please install Sass Compass.'));
  }
}

/**
 * Drush command callback; compiles Compass enabled modules in the current site.
 */
function drush_compass_compile() {
  $compass_options = array();
  $environment = drush_get_option('environment', 'production');
  $force = drush_get_option('force', FALSE);

  // Define command line options to pass to Compass.
  $compass_options[] = '--boring';
  $compass_options[] = '-e ' . filter_xss($environment);
  if ($force) {
    $compass_options[] = '--force';
  }

  // If there are no theme names specified in the arguments, then build an array
  // of all theme names from list_themes().
  $theme_names = func_get_args();
  if (count($theme_names) == 0) {
    $theme_names = array_map(function($theme) {
      return $theme->name;
    }, list_themes());
  }

  foreach ($theme_names as $theme_name) {
    $theme_path = drush_core_find_project_path($theme_name);

    // If the theme has a config.rb file it's Compass, if not carry on.
    if (!file_exists($theme_path . '/config.rb')) {
      continue;
    }

    $shell_cmd = sprintf('compass compile %s %s', escapeshellarg($theme_path), implode(' ', $compass_options));
    $exec_result = drush_shell_exec($shell_cmd);

    if ($exec_result) {
      drush_log(dt('@theme was compiled sucessfully ', array('@theme' => $theme_name)), 'success');
      if (drush_get_context('DRUSH_VERBOSE')) {
        drush_log(implode("\n", drush_shell_exec_output()));
      }
    }
    else {
      drush_set_error('COMPASS', dt("The theme @theme did not compile correctly\n@result", array('@theme' => $theme_name, '@result' => implode("\n", drush_shell_exec_output()))));
    }
  }
}
